<?php

namespace tests\Service\Manager\Mongo;


use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\Mongo\Mongo;
use Zeedhi\Framework\Util\NameProvider;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Zmart\QueryBuilders\Mongo\QueryBuilder;

class ManagerFindAggregateTest extends \PHPUnit_Framework_TestCase {
    const MONGO_DB_NAME = 'ZeedhiKPIUnitTest';
    const MONGO_PORT = '27017';
    const MONGO_IP = '192.168.122.55';
    const DATASOURCE_NAME = 'products';
    /** @var Manager */
    protected $manager;
    /** @var Mongo */
    protected static $mongoDB;

    public static function setUpBeforeClass() {
        if (!extension_loaded('mongo')) {
            self::markTestSkipped('The ' . __CLASS__ . ' requires the use of mongo extension.');
        }
        self::$mongoDB = new Mongo(self::MONGO_IP, self::MONGO_PORT, self::MONGO_DB_NAME);
    }

    public function setUp() {
        $ds = DIRECTORY_SEPARATOR;
        $slashes = $ds . '..' . $ds . '..' . $ds . '..' . $ds;
        $dataSourceDirectory = realpath(__DIR__) . $slashes . 'mocks' . $ds . 'gen' . $ds . 'datasources';
        $nameProvider = new NameProvider($dataSourceDirectory, "\\Model", array(), false);
        $this->manager = new Manager(self::$mongoDB, $nameProvider, new QueryBuilder());
        $rows = array(
            array(
                'product' => 'óculos top',
                'brand' => 'chillibeans',
                'price' => 1500.00
            ),
            array(
                'product' => 'óculos do carreta',
                'brand' => 'chillibeans',
                'price' => 100.00
            ),
            array(
                'product' => 'óculos do waguda',
                'brand' => 'chillibeans',
                'price' => 99.99
            ),
            array(
                'product' => 'óculos outra brand ruim',
                'brand' => 'triton',
                'price' => 20.00
            ),
            array(
                'product' => 'óculos outra brand maomeno',
                'brand' => 'triton',
                'price' => 30.00
            )
        );
        $this->manager->persist(new DataSet(self::DATASOURCE_NAME, $rows));
    }

    public function tearDown() {
        if (self::$mongoDB) {
            self::$mongoDB->dropCollection(self::DATASOURCE_NAME);
        }
    }

    public static function tearDownAfterClass() {
        if (self::$mongoDB) {
            self::$mongoDB->dropCollection(self::DATASOURCE_NAME);
        }
    }

    public function testFindAll() {
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos top',
                'brand' => 'chillibeans',
                'price' => 1500.00
            ),
            array(
                'product' => 'óculos do carreta',
                'brand' => 'chillibeans',
                'price' => 100.00
            ),
            array(
                'product' => 'óculos do waguda',
                'brand' => 'chillibeans',
                'price' => 99.99
            ),
            array(
                'product' => 'óculos outra brand ruim',
                'brand' => 'triton',
                'price' => 20.00
            ),
            array(
                'product' => 'óculos outra brand maomeno',
                'brand' => 'triton',
                'price' => 30.00
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testFindAllSorted() {
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME);
        $filterCriteria->addOrderBy('price', ZmartFilterCriteria::ORDER_DESC);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos top',
                'brand' => 'chillibeans',
                'price' => 1500.00
            ),
            array(
                'product' => 'óculos do carreta',
                'brand' => 'chillibeans',
                'price' => 100.00
            ),
            array(
                'product' => 'óculos do waguda',
                'brand' => 'chillibeans',
                'price' => 99.99
            ),
            array(
                'product' => 'óculos outra brand maomeno',
                'brand' => 'triton',
                'price' => 30.00
            ),
            array(
                'product' => 'óculos outra brand ruim',
                'brand' => 'triton',
                'price' => 20.00
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testFindAllLimited() {
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME);
        $filterCriteria->setPageSize(3);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos top',
                'brand' => 'chillibeans',
                'price' => 1500.00
            ),
            array(
                'product' => 'óculos do carreta',
                'brand' => 'chillibeans',
                'price' => 100.00
            ),
            array(
                'product' => 'óculos do waguda',
                'brand' => 'chillibeans',
                'price' => 99.99
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testFindAllLimitedSorted() {
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME);
        $filterCriteria->setPageSize(3);
        $filterCriteria->addOrderBy('price', ZmartFilterCriteria::ORDER_ASC);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos outra brand ruim',
                'brand' => 'triton',
                'price' => 20.00
            ),
            array(
                'product' => 'óculos outra brand maomeno',
                'brand' => 'triton',
                'price' => 30.00
            ),
            array(
                'product' => 'óculos do waguda',
                'brand' => 'chillibeans',
                'price' => 99.99
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testFindBy() {
        $conditions = array(
            array(
                'columnName' => 'brand',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'chillibeans'
            )
        );
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, $conditions);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos top',
                'brand' => 'chillibeans',
                'price' => 1500.00
            ),
            array(
                'product' => 'óculos do carreta',
                'brand' => 'chillibeans',
                'price' => 100.00
            ),
            array(
                'product' => 'óculos do waguda',
                'brand' => 'chillibeans',
                'price' => 99.99
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testFindByTwoConditions() {
        $conditions = array(
            array(
                'columnName' => 'brand',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'chillibeans'
            ),
            array(
                'columnName' => 'price',
                'operator' => ZmartFilterCriteria::LT,
                'value' => 1500.00
            )
        );
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, $conditions);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos do carreta',
                'brand' => 'chillibeans',
                'price' => 100.00
            ),
            array(
                'product' => 'óculos do waguda',
                'brand' => 'chillibeans',
                'price' => 99.99
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testFindBySorted() {
        $conditions = array(
            array(
                'columnName' => 'brand',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'chillibeans'
            )
        );
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, $conditions);
        $filterCriteria->addOrderBy('price', ZmartFilterCriteria::ORDER_DESC);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos top',
                'brand' => 'chillibeans',
                'price' => 1500.00
            ),
            array(
                'product' => 'óculos do carreta',
                'brand' => 'chillibeans',
                'price' => 100.00
            ),
            array(
                'product' => 'óculos do waguda',
                'brand' => 'chillibeans',
                'price' => 99.99
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testFindByBetween() {
        $conditions = array(
            array(
                'columnName' => 'price',
                'operator' => ZmartFilterCriteria::BETWEEN,
                'value' => array(200, 1700)
            )
        );
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, $conditions);
        $filterCriteria->addOrderBy('price', ZmartFilterCriteria::ORDER_DESC);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos top',
                'brand' => 'chillibeans',
                'price' => 1500.00
            )
        );

        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testFindByLimited() {
        $conditions = array(
            array(
                'columnName' => 'brand',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'chillibeans'
            )
        );
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, $conditions);
        $filterCriteria->setPageSize(1);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos top',
                'brand' => 'chillibeans',
                'price' => 1500.00
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testFindByLimitedSorted() {
        $conditions = array(
            array(
                'columnName' => 'brand',
                'operator' => ZmartFilterCriteria::EQ,
                'value' => 'chillibeans'
            )
        );
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, $conditions);
        $filterCriteria->setPageSize(2);
        $filterCriteria->addOrderBy('price', ZmartFilterCriteria::ORDER_ASC);
        $dataSet = $this->manager->findBy($filterCriteria);

        $rows = array(
            array(
                'product' => 'óculos do waguda',
                'brand' => 'chillibeans',
                'price' => 99.99
            ),
            array(
                'product' => 'óculos do carreta',
                'brand' => 'chillibeans',
                'price' => 100.00
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet->getDataSourceName(), $dataSet->getDataSourceName());
        $this->assertRows($expectedDataSet->getRows(), $dataSet->getRows(), array('_id'));
    }

    public function testAggregate() {
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, array());
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction('total_branch_prices', ZmartFilterCriteria::SUM, 'price');
        $dataSet = $this->manager->aggregate($filterCriteria);

        $rows = array(
            array(
                '_id' => 'triton',
                'total_branch_prices' => 50.00
            ),
            array(
                '_id' => 'chillibeans',
                'total_branch_prices' => 1699.99
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet, $dataSet);
    }

    public function testAggregateBetween() {
        $conditions = array(
            array(
                'columnName' => 'total_branch_prices',
                'operator' => ZmartFilterCriteria::BETWEEN,
                'value' => array(20, 1699.98)
            )
        );
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, array());
        $filterCriteria->setPostConditions($conditions);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction('total_branch_prices', ZmartFilterCriteria::SUM, 'price');
        $dataSet = $this->manager->aggregate($filterCriteria);

        $rows = array(
            array(
                '_id' => 'triton',
                'total_branch_prices' => 50.00
            )
        );

        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet, $dataSet);
    }

    public function testAggregateTwoFunctions() {
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, array());
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction('total_branch_prices', ZmartFilterCriteria::SUM, 'price');
        $filterCriteria->addAggregateFunction('avg_branch_prices', ZmartFilterCriteria::AVG, 'price');
        $dataSet = $this->manager->aggregate($filterCriteria);

        $rows = array(
            array(
                '_id' => 'triton',
                'total_branch_prices' => 50.00,
                'avg_branch_prices' => 25.00
            ),
            array(
                '_id' => 'chillibeans',
                'total_branch_prices' => 1699.99,
                'avg_branch_prices' => 566.6633333333333
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet, $dataSet);
    }

    public function testAggregateLimited() {
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, array());
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction('total_branch_prices', ZmartFilterCriteria::SUM, 'price');
        $filterCriteria->setPageSize(1);
        $dataSet = $this->manager->aggregate($filterCriteria);

        $rows = array(
            array(
                '_id' => 'triton',
                'total_branch_prices' => 50.00
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet, $dataSet);
    }

    public function testAggregateLimitedSorted() {
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, array());
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction('total_branch_prices', ZmartFilterCriteria::SUM, 'price');
        $filterCriteria->setPageSize(1);
        $filterCriteria->addOrderBy('total_branch_prices', ZmartFilterCriteria::ORDER_DESC);
        $dataSet = $this->manager->aggregate($filterCriteria);

        $rows = array(
            array(
                '_id' => 'chillibeans',
                'total_branch_prices' => 1699.99
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet, $dataSet);
    }

    public function testAggregateLimitedSortedHaving() {
        $conditions = array(
            array(
                'columnName' => 'total_branch_prices',
                'operator' => ZmartFilterCriteria::GTE,
                'value' => 1500.00
            )
        );
        $filterCriteria = new ZmartFilterCriteria(self::DATASOURCE_NAME, array());
        $filterCriteria->setPostConditions($conditions);
        $filterCriteria->addGroupBy('brand');
        $filterCriteria->addAggregateFunction('total_branch_prices', ZmartFilterCriteria::SUM, 'price');
        $filterCriteria->setPageSize(1);
        $filterCriteria->addOrderBy('total_branch_prices', ZmartFilterCriteria::ORDER_ASC);
        $dataSet = $this->manager->aggregate($filterCriteria);

        $rows = array(
            array(
                '_id' => 'chillibeans',
                'total_branch_prices' => 1699.99
            )
        );
        $expectedDataSet = new DataSet(self::DATASOURCE_NAME, $rows);
        $this->assertEquals($expectedDataSet, $dataSet);
    }

    public function assertRows($expectedRows, $rows, $ignoreKeys = array()) {
        foreach ($rows as &$row) {
            foreach ($ignoreKeys as $key) {
                unset($row[$key]);
            }
        }
        $this->assertEquals($expectedRows, $rows);
    }
}
 