import {Http, Loader, Metadata} from '@zeedhi/core';
import { Modal, ModalService, Tabs, Select, DialogService, Form, SelectMultiple, LoadingService } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { CustomAlertService } from '@/Services/CustomAlertService';
import { Utilities } from '@/Services/Utilities';
import { SkeletonMetadata } from '@/Services/SkeletonMetadata';
import {ChartViewFactory} from "@/Services/ChartView/ChartViewFactory";
import { ZmartChartsApex } from '../Services/Chats/ZmartChartsApex';

export class IndicatorsController {
    public modal!: Modal;

    public modalData: any = {};
    public indicatorData: any = {};
    public viewData: any = {};
    public goalData: any = {};
    public columnData: any = {};
    public templateData: any = {};
    public sugestedColors: string[] = [
        '#3cd7e7',
        '#7ac95b',
        '#f2ca27',
        '#ff7921',
        '#e74c3c',
        '#1ac3d6',
        '#5eb43c',
        '#f39c12',
        '#d35400',
        '#d62d1a',
        '#3498db',
        '#9b59b6',
        '#ff4c65',
        '#34495e',
        '#afb6bb',
        '#217dbb',
        '#804399',
        '#ff334f',
        '#22303d'
    ];

    // Container visibility
    public containersVisibility: any = {
        indicators: true,
        detail: false,
        view: false,
        goal: false,
        column: false,
    };

    public get indicatorsGridVisible() {
        return this.containersVisibility.indicators;
    }
    public get detailsVisible() {
        return this.containersVisibility.detail;
    }
    public get viewVisible() {
        return this.containersVisibility.view;
    }
    public get goalVisible() {
        return this.containersVisibility.goal;
    }
    public get columnVisible() {
        return this.containersVisibility.column;
    }

    public showContainer(containerName: string) {
        Object.keys(this.containersVisibility).forEach((key) => { this.containersVisibility[key] = false; });
        this.containersVisibility[containerName] = true;
    }

    // Modals
    public closeModal() {
        this.modal?.hide();
    }

    public async openIndicator() {
        this.modal?.destroy();
        this.modal = await ModalService.createFromJson('indicators/indicatorAdd', true);
        this.modal.show();
    }

    public async openView() {
        this.modal?.destroy();
        this.modal = await ModalService.createFromJson('indicators/view/viewDetail', true);
        this.modal.show();
    }

    public async openStyle({ row }: any) {
        this.modal?.destroy();
        this.modal = await ModalService.createFromJson('indicators/style/styleDetail', true);
        this.modalData = row || {};
        this.modal.show();
    }

    public async openGoalNotificationModal({ row }: any) {
        this.modal?.destroy();
        this.modal = await ModalService.createFromJson('indicators/view/goalSettings/goalNotificationDetail', true);
        this.modalData = row || {};
        this.modal.show();
    }

    public async openGoalModal({ row }: any) {
        this.modal?.destroy();
        this.modal = await ModalService.createFromJson('indicators/view/goalSettings/goalDetailAdd', true);
        this.modalData = row || {};
        this.goalData = {};
        this.modal.show();
    }

    // Form fill
    public async fillDetails() {
        const form = Metadata.getInstance<Form>("indicatorForm");
        form.value = this.indicatorData;
    }

    public hasDrillDown: boolean = false;
    public drillDownType: string = "";

    public get drilldownViewVisible() {
        return this.hasDrillDown && this.drillDownType === "widget";
    }

    public get drilldownDashboardVisible() {
        return this.hasDrillDown && this.drillDownType === "view";
    }

    public async fillView() {
        const form = Metadata.getInstance<Form>("viewForm");
        const dateFilterField = Metadata.getInstance<Select>("dateFilterColumn");
        const manualValueSetFields = ["period", "hasDrillDown", "drillDownType"];
        form.clearForm();
        Object.keys(this.viewData).filter((key) => !manualValueSetFields.includes(key)).forEach((key) => {
            form.value[key] = this.viewData[key];
        });
        this.hasDrillDown = this.viewData.hasDrillDown;
        this.drillDownType = this.viewData.drillDownType;
        this.generatePeriods(this.viewData.period);
        await dateFilterField.datasource.updateData(this.indicatorData.dataSourceColumns.filter((v: any) => v.isDate) || []);
        dateFilterField.setValue(this.viewData.dateFilterColumn);
    }

    private periodElementId: number = -1;
    public createPeriodElement(period: any = null, isLast: boolean = false) {
        this.periodElementId += 1;
        const periodFormat = Utilities.getDateFormatField(period, ' - ');
        let helperValue = "";
        if (typeof period === 'string') {
            helperValue = period;
        }
        return [{
            "name": "period_" + this.periodElementId,
            "component": "ZdDateRange",
            "label": "Período",
            "grid": { "cols": 7 },
            "value": periodFormat,
            "dateFormat": "DD/MM/YYYY",
            "inputFormat": "DD/MM/YYYY",
            "displayFormat": "DD/MM/YYYY",
            "splitter": " - ",
            "orderedDates": true,
            "helperOptions": this.periodData,
            "helperValue": helperValue,
            "events": { "onMounted": this.adjustPeriodDropdown.bind(this) }
        },
        {
            "name": "periodButtons_" + this.periodElementId,
            "component": "ZdContainer",
            "cssClass": "zd-pa-0",
            "children": [
                {
                    "name": "addButton_" + this.periodElementId,
                    "component": "ZdButton",
                    "label": "Adicionar outro periodo",
                    "iconName": "mdi-plus",
                    "isVisible": isLast,
                    "events": { "click": this.addPeriod.bind(this) }
                },
                {
                    "name": "deleteButton_" + this.periodElementId,
                    "component": "ZdButton",
                    "label": "Remover periodo",
                    "iconName": "mdi-delete",
                    "isVisible": !isLast,
                    "events": { "click": this.removePeriod.bind(this) }
                }
            ]
        }];
    }

    private periodData: any = [];
    public async generatePeriods(periods: any = [undefined]) {
        const form = Metadata.getInstance<Form>("viewForm_period");
        form.clearForm();
        if (!Array.isArray(periods)) {
            periods = [periods];
        }
        await this.generatePeriodHelperOptions();
        form.childrenInstances.forEach((el: any) => form.removeChild(el.name));
        form.children = [];
        form.childrenInstances = [];
        form.children = periods.map((period: any, i: number) => this.createPeriodElement(period, periods.length - 1 === i)).flat();
    }

    public adjustPeriodDropdown({ component }: any) {
        const value = component.helperValue;
        if (typeof value === 'string') {
            if (component.helperOptions?.includes(value)) {
                component.updateHelperHint(value);
            }
        }
        // Diminui o tamanho do menu de data
        const periodDropdown = component.childrenInstances[0].childrenInstances[0];
        periodDropdown.maxHeight = 190;
    }

    public async generatePeriodHelperOptions() {
        const periodEls = Metadata.getInstance<Form>("viewForm_period").children;

        periodEls.forEach((period: any) => {
            Utilities.unregisterPeriod(period);
        });

        this.periodData = await Utilities.getHelperOptions();
    }

    public updatePeriodStatus() {
        const form = Metadata.getInstance<Form>("viewForm_period");
        form.children.forEach((el: any, i: number) => {
            if (el.name.startsWith("periodButtons")) {
                if (i === form.children.length - 1) {
                    el.children[0].isVisible = true;
                    el.children[1].isVisible = false;
                } else {
                    el.children[0].isVisible = false;
                    el.children[1].isVisible = true;
                }
            }
        });
    }

    public addPeriod() {
        const form = Metadata.getInstance<Form>("viewForm_period");
        const period = this.createPeriodElement();
        form.addChild(period[0]);
        form.addChild(period[1]);
        this.updatePeriodStatus();
    }

    public removePeriod({ component }: any) {
        const form = Metadata.getInstance<Form>("viewForm_period");
        const id = component.name.replace("deleteButton_", "");
        form.removeChild("period_" + id);
        form.removeChild("periodButtons_" + id);
    }

    public getPeriodsValue() {
        const form = Metadata.getInstance<Form>("viewForm_period");
        let value = form.childrenInstances.filter((el: any) => el.name.startsWith("period_") && el).map((el: any) => el.helperValue || form.value[el.name]);
        value = value.filter((el: any) => !Array.isArray(el) || el.length === 2).map((el: any) => {
            if (Array.isArray(el)) {
                return { start: el[0], end: el[1] };
            }
            return el;
        });
        if (value.length === 1) {
            return value[0];
        }
        return value;
    }

    public async fillViewExtraForm() {
        const form = Metadata.getInstance<Form>("viewForm_extraFields");
        form.value = this.viewData;
    }

    public async fillStyle({ component }: any) {
        const styleFields = Metadata.getInstance<SelectMultiple>("styleFields");
        const style = Metadata.getInstance<Select>("styleId");
        styleFields.datasource.updateData(this.indicatorData.dataSourceColumns || []);
        style.setValue(this.modalData.styleId);
        component.value = this.modalData;
    }

    public async fillGoal() {
        const form = Metadata.getInstance<Form>("goalForm");
        form.value = this.goalData;
    }

    public async fillColumnConfig() {
        const form = Metadata.getInstance<Form>("columnConfigForm");
        form.value = this.columnData;
    }

    public get isColumnConfigFact() {
        return this.columnData.type === 'fact';
    }

    public async fillGoalNotification({ component }: any) {
        component.value = this.modalData;
    }

    // Show section
    public showDetails({ row }: any) {
        this.showContainer('detail');
        this.indicatorData = row || {};
        this.fillDetails();
        this.updateIndicatorFilter();
        this.updateViewFilter();
        this.updateStyleFilter();
        this.updateLogFilter();
    }

    public showIndicatorsGrid() {
        this.showContainer('indicators');
    }

    public returnToDetails() {
        this.showContainer('detail');
        Metadata.getInstance<TekGrid>("view-grid").reload();
    }

    public showView({ row }: any) {
        this.showContainer('view');
        this.viewData = row || {};
        this.updateColumnConfigFilter();
        this.updateGoalFilter();
        this.fillView();
        this.apexChartOnCreated();
        this.getViewFormFields();
        this.updateTabsDisable(!row);
    }

    public updateTabsDisable(isAdd: boolean) {
        const tabs = Metadata.getInstance<Tabs>("viewTabs");
        tabs.tabs[1].disabled = isAdd;
        tabs.tabs[2].disabled = isAdd;
        if (isAdd) {
            tabs.activeTab = 0;
        }
    }

    public updateViewType({component}: any) {
        if (component.datasource.currentRow) {
            this.templateData.template = component.datasource.currentRow.template;
            this.templateData.name = component.datasource.currentRow.name;
            this.templateData._id = component.datasource.currentRow._id;
            const type = component.value;
            if (type !== this.viewData.viewTypeId) {
                this.getViewFormFields(type);
            }
        }
    }

    public async getArrayColor() {
        const colorDataset = await Http.get("/color?page=1&limit=25");
        const color = this.sugestedColors;
        colorDataset.data.data.forEach((item: any) => {
            color.push(item.color);
        });
        return color;
    }

    public async getViewFormFields(viewTypeId: any = this.viewData.viewTypeId) {
        const chart = ChartViewFactory.getInstanceChartType(viewTypeId);
        const sugestedColors = await this.getArrayColor();
        this.viewData = chart.getEditData(this.viewData);
        const ZOrderCheck = (a: any, b: any) => {
            if (a.ZOrder && b.ZOrder) {
                return a.ZOrder - b.ZOrder;
            }
            return -1;
        };
        const formExtra = Metadata.getInstance<Form>("viewForm_extraFields");
        const viewTypeSelect = Metadata.getInstance<Select>("viewTypeId");
        formExtra.clearForm();
        formExtra.childrenInstances.forEach((el: any) => formExtra.removeChild(el.name));
        formExtra.children = [];
        formExtra.childrenInstances = [];

        let properties = viewTypeSelect.datasource.data.filter((item: any) => item._id === viewTypeId);
        properties = properties.length > 0 ? properties[0].properties : [];

        const res = await Metadata.get("indicators/view/viewDetailMega", true);
        const fieldsShown = res.children.filter((field: any) => properties.includes(field._id)).sort(ZOrderCheck);
        const groups: any = {};
        const noGroup: any = [];
        const groupsNames: any = res.ZFieldGroups;

        fieldsShown.forEach((field: any) => {
            if (field.datasource && field.datasource.filter && field.datasource.filter._id) {
                field.datasource.filter._id = this.indicatorData._id;
            }

            if (field.component === "ZdColorPicker") {
                field.swatches = sugestedColors;
            }

            if (field.ZFieldGroup) {
                if (!groups[field.ZFieldGroup]) {
                    groups[field.ZFieldGroup] = [];
                }

                groups[field.ZFieldGroup].push(field);
            } else {
                noGroup.push(field);
            }
        });

        const fieldsWithGroup: any = Object.entries(groups).map(([key, value]: [any, any]) => {
            const title = {
                "name": key + "_title",
                "component": "ZdText",
                "text": key in groupsNames ? groupsNames[key] : key,
                "tag": "h3",
                "grid": { "cols": 12 },
            };
            const divider = {
                "name": key + "_divider",
                "component": "ZdDivider",
                "grid": { "cols": 12 },
            };

            groups[key] = value.sort(ZOrderCheck);
            return [title, divider, ...groups[key]];
        });
        const fields = [...noGroup, ...([].concat(...fieldsWithGroup))];
        if (fields.length > 0) {
            fields.forEach((field: any) => {
                if (field.name in this.viewData) {
                    field.value = this.viewData[field.name];
                }
            });
        }
        formExtra.children = fields;
        // this.fillViewExtraForm();
    }

    public async apexChartOnCreated() {
        const component = Metadata.getInstance("viewChartCard");
        const viewId = this.viewData._id;
        this.buildSkeletonChart(component);
        await this.buildWidgetByViewId(component, viewId, {});
    }

    public buildSkeletonChart(component: any) {
        component.children = [
            SkeletonMetadata.skeletonIndicator().children[0]
        ];
    }

    public async onMountedInit() {
        const appController = Loader.getInstance('AppController');
        await appController.loginPromise.then();
        const gridIndicators = Metadata.getInstance<TekGrid>('indicators-grid');
        gridIndicators.reload();
    }

    public async buildWidgetByViewId(component: any, viewId: string, drillDownFilters: any) {
        if (!drillDownFilters) {
            drillDownFilters = {};
        }
        const params = {
            "requestType": "Row",
            "row": {
                "drillDownFilters": drillDownFilters,
                "serieColors": 0,
                "structureId": "",
                "structureName": "",
                "viewId": viewId
            }
        };
        const response = await Http.post("/buildWidgetByViewIdNext", params);
        const widget = response.data.dataset?.widget || null;
        component.children = [];
        if (widget == null) {
            component.children = [
                {
                    "name": "noData",
                    "component": "ZdText",
                    "text": "Não foi possível montar a visão"
                }
            ];
        } else {
            const charts = ZmartChartsApex.build(widget, this, component);
            const widgetChats = charts.getCharts();
            component.children = widgetChats;
        }
    }

    public returnToView() {
        this.showContainer('view');
    }

    public showGoal({ row }: any) {
        this.showContainer('goal');
        this.goalData = row || {};
        this.fillGoal();
        this.updateGoalNotificationFilter();
    }

    public showColumn({ row }: any) {
        this.showContainer('column');
        this.columnData = row || {};
        this.fillColumnConfig();
        this.updateFactColorFilter();
    }

    // Grid default filters
    public updateGridFilter(gridName: string, filterName: string, filterValue: any) {
        if (Metadata.getInstances(gridName).length === 0 || !Metadata.getInstances(gridName)[0]) {
            return;
        }
        const grid: any = Metadata.getInstance<TekGrid>(gridName);
        grid.datasource.addFilter(filterName, filterValue);
        grid.datasource.get();
    }

    public updateIndicatorFilter() {
        this.updateGridFilter('indicator-grid', 'indicator', this.indicatorData._id);
    }

    public updateViewFilter() {
        this.updateGridFilter('view-grid', 'indicator', this.indicatorData._id);
    }

    public updateStyleFilter() {
        this.updateGridFilter('style-grid', 'indicator', this.indicatorData._id);
    }

    public updateLogFilter() {
        this.updateGridFilter('log-grid', 'collectionId', this.indicatorData._id);
    }

    public updateColumnConfigFilter() {
        this.updateGridFilter('columnConfig-grid', 'widget', this.viewData._id);
    }

    public updateGoalFilter() {
        this.updateGridFilter('goalSettings-grid', 'viewId', this.viewData._id);
    }

    public updateGoalNotificationFilter() {
        this.updateGridFilter('goalNotification-grid', 'goalId', this.goalData._id);
    }

    public updateFactColorFilter() {
        if (Metadata.getInstances('factColor-grid').length === 0 || !Metadata.getInstances('factColor-grid')[0]) {
            return;
        }
        const grid: any = Metadata.getInstance<TekGrid>('factColor-grid');
        grid.datasource.setFilter({
            'viewId': this.viewData._id,
            'dimension': this.viewData.serieFieldLabel,
            'structureId': [this.viewData.viewTypeId]
        });
        grid.datasource.get();
    }

    public inlineEditColor: any = {};
    public hexColorChange({ row, component }: any) {
        if (!row) {
            return;
        }
        const grid = Metadata.getInstance<TekGrid>('factColor-grid');
        const value = component.value;
        this.inlineEditColor[row._id] = value;
        grid.reapplyConditions(row);
    }

    public getHexadecimalColor({ row }: any) {
        const color = this.inlineEditColor[row._id] ? this.inlineEditColor[row._id] : row.color;
        return { backgroundColor: color, width: '100%' };
    }

    // Form saves
    public async saveIndicator() {
        let form = Metadata.getInstance<Form>("indicatorForm");
        if (Metadata.getInstances("indicatorFormAdd").length > 0 && Metadata.getInstances("indicatorFormAdd")[0]) {
            form = Metadata.getInstance<Form>("indicatorFormAdd");
        }
        // Informar caso o campo de dataSource não esteja preenchido? (select não é validado automaticamente)
        if (!form.validate()) {
            return;
        }

        const { menu, dataSource } = form.value;
        const values = {
            ...form.value,
            dataSourceColumns: dataSource?.columns || null,
            dataSourceName: dataSource?.name || null,
            dataSource: dataSource?._id || null,
            menuName: menu?.name || null,
            menu: menu?._id || null
        };
        try {
            LoadingService.show();
            const res = await Http.post("/indicator/save", {
                requestType: "DataSet",
                disableLoader: false,
                dataset: [values]
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            this.reloadIndicatorGrid();
            this.closeModal();
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    public prepareWidgetToSave(formExtra: any, values: any, cleanValues: any) {
        if (values.hasDrillDown || values.drillDownType === 'structure') {
            cleanValues.drillDownViewField = null;
            cleanValues.drillDownView = null;
            cleanValues.drillDownDashboardField = null;
        } else if (values.hasDrillDown) {
            if (values.drillDownType === 'widget') {
                cleanValues.drillDownDashboardField = null;
            } else if (values.drillDownType === 'view') {
                cleanValues.drillDownViewField = null;
                cleanValues.drillDownView = null;
            }
        }

        cleanValues.template = this.templateData.template;
        cleanValues.chartType = this.templateData.name;
        cleanValues.viewType = this.templateData.name;
        cleanValues.serieType = this.templateData.name;
        cleanValues.columns = this.indicatorData.dataSourceColumns;
        const chart = ChartViewFactory.getInstanceChartType(cleanValues.viewTypeId);
        chart.setEditData(this.viewData);
        cleanValues = chart.getSaveViewParams(formExtra, values, cleanValues);
        return cleanValues;
    }

    public async saveView() {
        const form = Metadata.getInstance<Form>("viewForm");
        const formExtra = Metadata.getInstance<Form>("viewForm_extraFields");
        const period = this.getPeriodsValue();
        let isNew = false;

        if (!form.validate() || !formExtra.validate()) {
            return;
        }

        const values: any = {
            ...form.value,
            ...formExtra.value,
            period
        };
        if (!("indicator" in values)) {
            values.indicator = this.indicatorData._id;
            isNew = true;
        }
        const cleanValues = Object.fromEntries(Object.entries(values).filter(([key, value]: [string, any]) => value != null));
        this.prepareWidgetToSave(formExtra, values, cleanValues);
        try {
            LoadingService.show();
            const res = await Http.post("/widgetsWithIndicatorNames/save", {
                requestType: "DataSet",
                disableLoader: false,
                dataset: [cleanValues],
            });
            this.viewData = cleanValues;
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            this.reloadIndicatorGrid();
            this.apexChartOnCreated();
            if (isNew) {
                const id = res.data.dataset.widget[0];
                this.viewData._id = id;
                this.updateTabsDisable(false);
            }
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    public async saveGoalNotification() {
        const form = Metadata.getInstance<Form>("goalNotificationForm");
        if (!form.validate()) {
            return;
        }
        const values = {
            ...form.value,
            goalId: this.goalData._id
        };
        try {
            LoadingService.show();
            const res = await Http.post("/goalNotifications/save", {
                requestType: "DataSet",
                disableLoader: false,
                dataset: [values]
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            this.reloadGoalsNotificationGrid();
            this.closeModal();
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    public async saveGoal() {
        let form = Metadata.getInstance<Form>("goalForm");
        if (Metadata.getInstances("goalFormAdd").length > 0 && Metadata.getInstances("goalFormAdd")[0]) {
            form = Metadata.getInstance<Form>("goalFormAdd");
        }
        if (!form.validate()) {
            return;
        }
        const values = {
            ...form.value,
            viewId: this.viewData._id
        };
        try {
            LoadingService.show();
            const res = await Http.post("/goals/save", {
                requestType: "DataSet",
                disableLoader: false,
                dataset: [values]
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            this.reloadGoalsGrid();
            this.closeModal();
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    public async saveStyle() {
        const form = Metadata.getInstance<Form>("styleForm");
        const style = form.value.styleId;
        const values = {
            ...form.value,
            cssClass: style?.class,
            styleId: style?._id,
            styleName: style?.name,
            indicator: this.indicatorData._id
        };
        try {
            LoadingService.show();
            const res = await Http.post("/gridStyle/save", {
                requestType: "DataSet",
                disableLoader: false,
                dataset: [values]
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            if (res.data.messages?.length > 0) {
                const message = res.data.messages[0].message;
                DialogService.show({
                    name: 'warning-dialog',
                    type: 'error',
                    title: ' ',
                    text: message
                });
            }
            this.reloadStyleGrid();
            this.closeModal();
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    // Inline edits
    public async saveIndicatorInline(params: any) {
        Utilities.genericBeforeSaveTekGrid(params, "indicator/save", "indicators-grid");
    }

    public deleteWidget(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/widget/remove", "view-grid");
    }

    public deleteIndicator(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/indicator/remove", "indicators-grid");
    }

    public async saveStyleInline(params: any) {
        Utilities.genericBeforeSaveTekGrid(params, "gridStyle/save", "style-grid");
    }

    public deleteStyle(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/gridStyle/remove", "style-grid");
    }

    public async saveColumnConfigInline(params: any) {
        Utilities.genericBeforeSaveTekGrid(params, "columnsWidget/save", "columnConfig-grid");
    }

    public async saveGoalInline(params: any) {
        Utilities.genericBeforeSaveTekGrid(params, "goals/save", "goalSettings-grid");
    }

    public deleteGoal(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/goals/remove", "goalSettings-grid", "Row");
    }

    public deleteGoalNotification(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/goalNotifications/remove", "goalNotification-grid", "Row");
    }

    // Grid reloads
    public reloadStyleGrid() {
        const grid = Metadata.getInstance<TekGrid>("style-grid");
        grid.reload();
    }

    public reloadIndicatorGrid() {
        const grid = Metadata.getInstance<TekGrid>("indicators-grid");
        grid.reload();
    }

    public reloadGoalsGrid() {
        const grid = Metadata.getInstance<TekGrid>("goalSettings-grid");
        grid.reload();
    }

    public reloadGoalsNotificationGrid() {
        const grid = Metadata.getInstance<TekGrid>("goalNotification-grid");
        grid.reload();
    }
}
